	SUBROUTINE find_variables

*	List all variables to be generated.
*	A small Finite State Machine (FMS) with 2 internal states
*	implements the search for variables on each line:
* 	status 1 = searching for first variable's delimiter (starter)
*	status 2 = searching for second variable's delimiter (terminator).
*	More variables can stay on the same text line.
*	Only one variable can stay on each range line.
*	(COMPITO package)



	IMPLICIT NONE
	SAVE


	INTEGER i,j		! loop indices
	INTEGER n1, n2		! string pointers
	INTEGER joff		! character offset on text line, string from
				! character #1 to "joff" was already examined
	INTEGER jpoi		! character pointer on text line
	INTEGER jpoia           ! Aux pointer
        INTEGER jpo2            ! second pointer to handle literal answers
	INTEGER istat		! internal status of FSM
	INTEGER iret		! return code from system routines
	INTEGER lvar		! variable's name length
	CHARACTER*100 ran_var	! variable's name

	LOGICAL found

	INCLUDE 'compito.inc'
	
	INTEGER jp1(max_var)	! pointer to first character in "range"
	INTEGER jp2(max_var)	! pointer to last character in "range"
	INTEGER jr_var(max_var)	! range number corresponding to a given variable

*	External functions

	INTEGER STRUPCASE	! Convert to uppercase (Run-time library)


*	Executable code


*	initialize counter of variables
	nvar = 0
        jpo2 = 0
	

	  DO 30000, i = 1, last_text		! loop on text lines


*	  initialize FSM
	  joff = 0
	  istat = 1
        

*	  jump to appropriate code for current FSM's internal state

100	  CONTINUE

	  GO TO (10000, 20000)  istat

	  PRINT *, ' '
	  PRINT *, ' FIND_VARIABLES: Wrong internal status istat:', istat
	  PRINT *, ' '
	  CALL LIBSTOP (%VAL(ss_abort))


*	  searching for variable's starter

10000	  CONTINUE

*	  check end of line
	  IF (joff .GE. ltex(i))  GO TO 30000

*	  pointer to starter
	  jpoia = jpoi
	  jpoi = INDEX (text(i)(joff+1:ltex(i)), var_delimiter)
C
c	  IF (jpoi.GT.1) THEN
c	     IF (text(i)(jpoi-1:jpoi-1).EQ."a") THEN 
c		jpoi = INDEX (text(i)(jpoi+1:ltex(i)), var_delimiter)
c	     ENDIF
c	  ENDIF
          jpo2 = MAX (jpo2, INDEX(text(i), literal_answer))

	    IF (jpoi .NE. 0)  THEN

	      nvar = nvar + 1

	        IF (nvar .GT. max_var)  THEN

	          PRINT *, ' '
	          PRINT *, ' FIND_VARIABLES: too many variables found'
	          PRINT *, ' maximum allowed (max_var):', max_var
	          PRINT *, ' exceeding variable found on text line:', i
	          PRINT *, ' ', text(i)(1:ltex(i))
	          PRINT *, ' '
	          CALL LIBSTOP (%VAL(ss_abort))

	        END IF

	      joff = joff + jpoi
	      jfirst(nvar) = joff + 1
	      istat = 2				! set FSM status
	      GO TO 100

	    ELSE

	      GO TO 30000

	    END IF


*	  searching for variable's terminator

20000	  CONTINUE

*	  check end of line

	    IF (joff .GE. ltex(i))  THEN

	      PRINT *, ' '
	      PRINT *, ' FIND_VARIABLES: variable terminator not found (1)'
	      PRINT *, ' text line:', i, ' character:',  jfirst(nvar)-1
	      PRINT *, text(i)(1:ltex(i))
	      PRINT *, ' '
	      CALL LIBSTOP (%VAL(ss_abort))

	    END IF

*	  pointer to terminator
	  jpoi = INDEX (text(i)(joff+1:ltex(i)), var_delimiter)
	    
	    IF (jpoi .EQ. 0)  THEN

	      PRINT *, ' '
	      PRINT *, ' FIND_VARIABLES: variable terminator not found (2)'
	      PRINT *, ' text line:', i, ' character:',  jfirst(nvar)-1
	      PRINT *, text(i)(1:ltex(i))
	      PRINT *, ' '
	      CALL LIBSTOP (%VAL(ss_abort))

	    ELSE
C Added by RM to handle embedded var_delimiter

	      linevar(nvar) = i
	      jlast(nvar) = joff + jpoi - 1
	      lv(nvar) = jlast(nvar) - jfirst(nvar) + 1

*	        check name length

	        IF (lv(nvar) .LE. 0)  THEN

	          PRINT *, ' '
	          PRINT *, ' FIND_VARIABLES: no character in variable name'
	          PRINT *, ' variable', nvar, ' on text line', linevar(nvar)
	          PRINT *, text(linevar(nvar))(1:ltex(linevar(nvar)))
	          PRINT *, ' '
	          CALL LIBSTOP (%VAL(ss_abort))

	        ELSE IF (lv(nvar) .GT. LEN(var(1)))  THEN

	          PRINT *, ' '
	          PRINT *, ' FIND_VARIABLES: name of variable too long'
	          PRINT *, ' maximum allowed length:', LEN(var(1))
	          PRINT *, ' wrong variable is number', nvar
	          PRINT *, ' on text line', linevar(nvar)
	          PRINT *, text(linevar(nvar))(1:ltex(linevar(nvar)))
	          PRINT *, ' '
	          CALL LIBSTOP (%VAL(ss_abort))

	        END IF

	      var(nvar)(1:lv(nvar)) = text(i)(jfirst(nvar):jlast(nvar))
	      sav(nvar) = var(nvar)		! save original case
	      iret = STRUPCASE (var(nvar), var(nvar), len(var(nvar)))	! convert to uppercase
	      joff = joff + jpoi

*	      check name

	      jpoi = INDEX (var(nvar)(1:lv(nvar)), ' ')

	        IF (jpoi .NE. 0)  THEN

	          PRINT *, ' '
	          PRINT *, ' FIND_VARIABLES: blank character in variable name'
	          PRINT *, ' variable', nvar, ' on text line', linevar(nvar)
	          PRINT *, text(linevar(nvar))(1:ltex(linevar(nvar)))
	          PRINT *, ' '
	          CALL LIBSTOP (%VAL(ss_abort))

	        END IF

	      istat = 1					! set FSM status
	      GO TO 100
	
	    END IF


30000	  CONTINUE				! end loop on text lines


*	  check number of random variables

	  IF (nvar .LE. 0 .AND. jpo2 .EQ. 0)  THEN

	    PRINT *, ' '
	    PRINT *, ' FIND_VARIABLES: no random variables found in input text!'
	    PRINT *, ' '
	    PRINT *, '       ***           WARNING  !           ***'
	    PRINT *, ' '

	  END IF


*	  check that no variable name is duplicated

	  DO i = 1, nvar-1
	  DO j = i+1, nvar

	    IF (var(i)(1:lv(i)) .EQ. var(j)(1:lv(j)))  THEN

	      PRINT *, ' '
	      PRINT *, ' FIND_VARIABLES: duplicate name ', var(i)(1:lv(i))
	      PRINT *, ' text line', linevar(i), ', character', jfirst(i)
	      PRINT *, text(linevar(i))(1:ltex(linevar(i)))
	      PRINT *, ' text line', linevar(j), ', character', jfirst(j)
	      PRINT *, text(linevar(j))(1:ltex(linevar(j)))
	      PRINT *, ' '
	      CALL LIBSTOP (%VAL(ss_abort))

	    END IF

	  END DO
	  END DO


*	  get definitions of ranges


	  DO 50000, i = 1, last_range		! loop on range lines


*	  variable's first pointer
	  jpoi = INDEX (range(i)(1:lran(i)), var_delimiter)

	    IF (jpoi .EQ. 0 .AND. jpo2 .EQ. 0)  THEN

	      PRINT *, ' '
	      PRINT *, ' FIND_VARIABLES: no variable found on RANGE line', i
	      PRINT *, ' ', range(i)(1:lran(i))
	      PRINT *, ' variable delimiter is (var_delimiter) ', var_delimiter
	      PRINT *, ' '
	      CALL LIBSTOP (%VAL(ss_abort))

	    END IF

	  jp1(i) = jpoi + 1

*	  variable's last pointer
	  jpoi = INDEX (range(i)(jp1(i):lran(i)), var_delimiter)

	    IF (jpoi .EQ. 0 .AND. jpo2 .EQ. 0)  THEN

	      PRINT *, ' '
	      PRINT *, ' FIND_VARIABLES: no variable found on RANGE line', i
	      PRINT *, ' ', range(i)(1:lran(i))
	      PRINT *, ' variable delimiter is (var_delimiter) ', var_delimiter
	      PRINT *, ' variable''s terminator not seen'
	      PRINT *, ' '
	      CALL LIBSTOP (%VAL(ss_abort))

	    END IF

	  jp2(i) = jpoi + jp1(i) - 2

*	    check line length

	    IF (lran(i) .LT. (jp2(i) + 2))  THEN

	      PRINT *, ' '
	      PRINT *, ' FIND_VARIABLES: RANGE line', i, ' is too short'
	      PRINT *, ' ', range(i)(1:lran(i))
	      PRINT *, ' '
	      CALL LIBSTOP (%VAL(ss_abort))

	    END IF


50000	  CONTINUE				! end of loop on range lines


*	check that all variables are declared in the range definition


*	  first check number of ranges

	  IF (last_range .NE. nvar)  THEN

	    PRINT *, ' '
	    PRINT *, ' FIND_VARIABLES: mismatch on number of random variables'
	    PRINT *, ' number of variables found in TEXT  section:', nvar
	    PRINT *, ' number of variables found in RANGE section:', last_range
	    PRINT *, ' '
	    CALL LIBSTOP (%VAL(ss_abort))

	  END IF


*	  then check variables' names

	  DO i = 1, nvar				! loop on variables

	  found = .FALSE.

	    DO j = 1, last_range			! loop on ranges

	    lvar = jp2(j) - jp1(j) + 1
	    iret = STRUPCASE (ran_var(1:lvar), range(j)(jp1(j):jp2(j)),
     &                    len(range(j)(jp1(j):jp2(j))))

	      IF (var(i)(1:lv(i)) .EQ. ran_var(1:lvar))  THEN

	        found = .TRUE.				! set flag
	        jr_var(i) = j				! store range number

	      END IF

	    END DO


	    IF (.NOT. found)  THEN

	      PRINT *, ' '
	      PRINT *, ' FIND_VARIABLE: text-range mismatch'
	      PRINT *, ' variable number', i, ': ', var(i)(1:lv(i))
	      PRINT *, ' found in TEXT line', linevar(i)
	      PRINT *, ' ', text(linevar(i))(1:ltex(linevar(i)))
	      PRINT *, ' was not found in RANGE definition'
	      PRINT *, ' '
	      CALL LIBSTOP (%VAL(ss_abort))

	    END IF

	  END DO					! end loop on variables


*	  store ranges

	  DO i = 1, nvar

	    j = jr_var(i)
	    n1 = jp2(j) + 2
	    n2 = lran(j)
c	    READ (range(j)(n1:n2), *), rlow(i), rup(i)
	    READ (range(j)(n1:n2), *) rlow(i), rup(i)

	  END DO


	RETURN

	END
