*	FILE: Parameters.inc
*
*	Include file for package COMPITO
*

*	system messages

	INTEGER ss_abort			! see module $SSDEF
	PARAMETER (ss_abort = '0000002C'X)


*	size parameters

	INTEGER max_line			! maximum number of text lines
	PARAMETER (max_line = 5000)

	INTEGER max_var				! maximum number of variables
	PARAMETER (max_var = 100)

	INTEGER max_question			! maximum number of questions
	PARAMETER (max_question = 40)

	INTEGER max_random_retry		! maximum number of retries
	PARAMETER (max_random_retry = 100)	! for random extraction

	INTEGER max_constraint			! maximum number of constraints
	PARAMETER (max_constraint = 50)

	INTEGER max_attempt			! maximum number of attempts in
	PARAMETER (max_attempt = 50)		! generating an alternative

	INTEGER max_format			! maximum size for F formats
	PARAMETER (max_format = 7)

        INTEGER max_forced			! maximum number of forced 
        PARAMETER (max_forced = 25)		! answers 

*	number of alternative answers proposed to the student,
*	including the correct answer
	INTEGER n_alternatives
	PARAMETER (n_alternatives = 5)

	REAL precision			! required precision
	PARAMETER (precision = 0.01)	! 2% precision

	REAL gap			! minimum fractional difference
	PARAMETER (gap = 2.*precision)	! between proposed answers

	REAL scale_down			! re-scaling factor applied to solution
	PARAMETER (scale_down = 0.1)	! for the alternative lower edge

	REAL scale_up			! re-scaling factor applied to solution
	PARAMETER (scale_up = 3.0)	! for the alternative upper edge

C Added by isidoro.ferrante@pi.infn.it 28/3/2000
C to generate the answers in a better range
        REAL exprange                   ! order of magnitude range for 
        PARAMETER (exprange = 1.5)      ! the alternative answers

*	control characters

*	NOTE: for the end-of-section control words the declared length
*	      MUST match number of characters in PARAMETER statement:
*	      change corresponding declaration when changing control word
*
*	NOTE: end-of-section control words MUST be UPPERCASE

*	end of TEXT section control word
	CHARACTER*8 end_text
	PARAMETER (end_text = 'END_TEXT')

*	end of RANGE section control word
	CHARACTER*9 end_range
	PARAMETER (end_range = 'END_RANGE')

*	end of SOLUTION section control word
	CHARACTER*12 end_solve
	PARAMETER (end_solve = 'END_SOLUTION')

*	end of CONSTRAINT section control word
	CHARACTER*14 end_cons
	PARAMETER (end_cons = 'END_CONSTRAINT')
	
*       end of FORCED section control word
        CHARACTER*10 end_forc
        PARAMETER (end_forc = 'END_FORCED')

*	answer line control words
*	NOTE: the declared length MUST match the actual number of characters
*	      in the PARAMETER statements: change the corresponding declaration
*	      if control word is changed
*	NOTE: the words MUST be UPPERCASE
	CHARACTER*7 answer_line
	PARAMETER (answer_line = 'ANSWERS')

	CHARACTER*4 unstable_sign
	PARAMETER (unstable_sign = 'FLIP')

C Added by isidoro.ferrante@pi.infn.it 28/3/2000
C flag to generate INT answers
        CHARACTER*3 integer_answer
        PARAMETER (integer_answer = 'INT')

C Added by riccardo.mannella@df.unipi.it 6/12/2000
C to generate analitycal answers
        CHARACTER*7 literal_answer
        PARAMETER (literal_answer = 'LITERAL')

C Added by riccardo.mannella@df.unipi.it 12/1/2004
C to generate box for formula and numerical results
        CHARACTER*7 both_answer
        PARAMETER (both_answer = 'BOTH')

*	Physics & Math

	REAL pi
	PARAMETER (pi = 3.1415927)

	REAL twopi
	PARAMETER (twopi = 2.*pi)

	REAL rad_to_deg
	PARAMETER (rad_to_deg = 180./pi)

	REAL deg_to_rad
	PARAMETER (deg_to_rad = pi/180.)

	REAL cm_to_m
	PARAMETER (cm_to_m = 0.01)

	REAL m_to_cm
	PARAMETER (m_to_cm = 100.)

	REAL g_to_Kg
	PARAMETER (g_to_Kg = 0.001)

	REAL sec_to_hour
	PARAMETER (sec_to_hour = 1./3600.)

	REAL m_sec_to_km_h
	PARAMETER (m_sec_to_km_h = 3.6)

	REAL km_h_to_m_sec
	PARAMETER (km_h_to_m_sec = 1./3.6)

	REAL c_light
	PARAMETER (c_light = 2.9979246E+8)	! m/sec

	REAL mu0
	PARAMETER (mu0 = 4.*pi*1.E-7)		! H/m

	REAL epsi0
	PARAMETER (epsi0 = 1./(mu0*c_light*c_light))

C	REAL epsi0
C	PARAMETER (epsi0 = 0.885E-11)		! F/m
C
C	REAL mu0
C	PARAMETER (mu0 = 0.126E-5)		! H/m

	REAL N_Avogadro
	PARAMETER (N_Avogadro = 6.02E+23)	! mole**(-1)

	REAL R_gas
	PARAMETER (R_gas = 8.31)		! J/(mole*K)

	REAL g_earth
	PARAMETER (g_earth = 9.81)		! m/sec**2

	REAL G_gravity
	PARAMETER (G_gravity = 6.67E-11)	! N*m**2*kg**-2

	REAL M_earth
	PARAMETER (M_earth = 5.98E+24)		! kg



*       letters

        CHARACTER*1 letter(30)
        DATA letter(1)  / 'A' /
        DATA letter(2)  / 'B' /
        DATA letter(3)  / 'C' /
        DATA letter(4)  / 'D' /
        DATA letter(5)  / 'E' /
        DATA letter(6)  / 'F' /
        DATA letter(7)  / 'G' /
        DATA letter(8)  / 'H' /
        DATA letter(9)  / 'I' /
        DATA letter(10) / 'J' /
        DATA letter(11) / 'K' /
        DATA letter(12) / 'L' /
        DATA letter(13) / 'M' /
        DATA letter(14) / 'N' /
        DATA letter(15) / 'O' /
        DATA letter(16) / 'P' /
        DATA letter(17) / 'Q' /
        DATA letter(18) / 'R' /
        DATA letter(19) / 'S' /
        DATA letter(20) / 'T' /
        DATA letter(21) / 'U' /
        DATA letter(22) / 'V' /
        DATA letter(23) / 'W' /
        DATA letter(24) / 'X' /
        DATA letter(25) / 'Y' /
        DATA letter(26) / 'Z' /


